#include <gtk/gtk.h>

enum OP
{ NOP, ADD, SUB, MUL, DIV };

typedef struct
{
  GtkWidget *win;
  GtkWidget *digit[9];
  GtkWidget *add, *sub, *mul, *div, *res;
  GtkWidget *vbox, *grid;
  GtkWidget *ent;
  gint acc;
  gint op;
  gint save;
  gboolean status;
} WinData;

gboolean
on_close (GtkWidget * gwt, gpointer data)
{
  gtk_main_quit ();
  return TRUE;
}


void
save_status (gpointer data)
{
  WinData *wData = data;
  switch (wData->op)
    {
    case ADD:
      wData->save = wData->save + wData->acc;
      break;
    case SUB:
      wData->save = wData->save - wData->acc;
      break;
    case MUL:
      wData->save = wData->save * wData->acc;
      break;
    case DIV:
      wData->save = wData->save / wData->acc;
      break;
    default:
      wData->save = wData->acc;
    }
  wData->status = 0;
  wData->op = NOP;

}



gboolean
on_digit (GtkWidget * gwt, gpointer data)
{
  WinData *wData = data;
  if (wData->status == 1)
    {
      wData->save = wData->acc;
      wData->acc = 0;
    }
  const gchar *lab = gtk_button_get_label (GTK_BUTTON (gwt));
  gchar *buf_acc = g_strdup_printf ("%i%s", wData->acc, lab);
  gint newVal = g_ascii_strtod (buf_acc, NULL);
  wData->acc = newVal;
  buf_acc = g_strdup_printf ("%i", wData->acc);
  gtk_entry_set_text (GTK_ENTRY (wData->ent), buf_acc);
  wData->status = 0;
}


gboolean
on_add (GtkWidget * gwt, gpointer data)
{
  WinData *wData = data;
  wData->op = ADD;
  wData->save = wData->acc;
  wData->status = 1;
}

gboolean
on_sub (GtkWidget * gwt, gpointer data)
{
  WinData *wData = data;
  wData->op = SUB;
  wData->save = wData->acc;
  wData->status = 1;
}

gboolean
on_mul (GtkWidget * gwt, gpointer data)
{
  WinData *wData = data;
  wData->op = MUL;
  wData->save = wData->acc;
  wData->status = 1;
}

gboolean
on_div (GtkWidget * gwt, gpointer data)
{
  WinData *wData = data;
  wData->op = DIV;
  wData->save = wData->acc;
  wData->status = 1;
}


gboolean
on_res (GtkWidget * gwt, gpointer data)
{
  WinData *wData = data;

  save_status (data);



  gchar *buf_acc = g_strdup_printf ("%i", wData->save);
  gtk_entry_set_text (GTK_ENTRY (wData->ent), buf_acc);
  wData->acc = wData->save;
  wData->save = 0;
  wData->op = NOP;
}


int
main (int argc, char *argv[])
{
  WinData wData;
  wData.acc = 0;
  wData.op = NOP;
  wData.save = 0;
  wData.status = 0;

  gtk_init (&argc, &argv);
  wData.win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (wData.win), "L2 Calc");
  gtk_container_set_border_width (GTK_CONTAINER (wData.win), 10);

  wData.vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
  wData.grid = gtk_grid_new ();
  gtk_container_add (GTK_CONTAINER (wData.win), wData.vbox);
  g_signal_connect (G_OBJECT (wData.win), "delete_event",
		    G_CALLBACK (on_close), &wData);

  wData.ent = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (wData.vbox), wData.ent, TRUE, TRUE, 5);
  gtk_box_pack_start (GTK_BOX (wData.vbox), wData.grid, TRUE, TRUE, 5);

  // Les boutons digits 
  int cmp = 1;
  for (int i = 0; i <= 3; i++)
    {
      for (int j = 0; j <= 2; j++)
	{
	  gchar *lab = g_strdup_printf ("%i", cmp % 10);
	  wData.digit[cmp] = gtk_button_new_with_label (lab);

	  // A grandir pour occuper tout l'espace en H et V
	  gtk_widget_set_hexpand (wData.digit[cmp], TRUE);
	  gtk_widget_set_vexpand (wData.digit[cmp], TRUE);

	  gtk_grid_attach (GTK_GRID (wData.grid), wData.digit[cmp], j, i, 1,
			   1);
	  g_signal_connect (G_OBJECT (wData.digit[cmp]), "clicked",
			    G_CALLBACK (on_digit), &wData);
	  cmp++;
	  // On arrête dès qu'on dessine les 10 boutons 
	  if (cmp > 10)
	    {
	      break;
	    }
	}
    }


  // Les autres boutons 
  wData.add = gtk_button_new_with_label ("+");
  gtk_grid_attach (GTK_GRID (wData.grid), wData.add, 3, 0, 1, 1);
  g_signal_connect (G_OBJECT (wData.add), "clicked", G_CALLBACK (on_add), &wData);

  wData.sub = gtk_button_new_with_label ("-");
  gtk_grid_attach (GTK_GRID (wData.grid), wData.sub, 3, 1, 1, 1);
  g_signal_connect (G_OBJECT (wData.sub), "clicked", G_CALLBACK (on_sub), &wData);

  wData.mul = gtk_button_new_with_label ("*");
  gtk_grid_attach (GTK_GRID (wData.grid), wData.mul, 3, 2, 1, 1);
  g_signal_connect (G_OBJECT (wData.mul), "clicked", G_CALLBACK (on_mul), &wData);

  wData.div = gtk_button_new_with_label ("/");
  gtk_grid_attach (GTK_GRID (wData.grid), wData.div, 3, 3, 1, 1);
  g_signal_connect (G_OBJECT (wData.div), "clicked", G_CALLBACK (on_div), &wData);

  wData.res = gtk_button_new_with_label ("=");
  gtk_grid_attach (GTK_GRID (wData.grid), wData.res, 1, 3, 2, 1);
  g_signal_connect (G_OBJECT (wData.res), "clicked", G_CALLBACK (on_res), &wData);

  gtk_widget_show_all (wData.win);
  gtk_main ();
  return 0;
}
